﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Web.Mvc;
using System.Web.Routing;

namespace org.theGecko.Utilities.MefMvc
{
    /// <summary>
    /// Based on ideas from http://blog.maartenballiauw.be/post/2009/04/21/ASPNET-MVC-and-the-Managed-Extensibility-Framework-%28MEF%29.aspx
    /// </summary>
    public class MefControllerFactory : IControllerFactory
    {
        private readonly MefContext _context;
        private readonly DefaultControllerFactory _defaultFactory;

        public MefControllerFactory(MefContext context)
        {
            _context = context;
            _defaultFactory = new DefaultControllerFactory();
        }

        #region IControllerFactory Members

        public IController CreateController(RequestContext requestContext, string controllerName)
        {
            IController controller = SelectController(requestContext, controllerName);

            // Inject extra context
            if (controller is MefController)
            {
                (controller as MefController).Context = _context;
            }

            return controller;
        }

        private IController SelectController(RequestContext requestContext, string controllerName)
        {
            if (controllerName != null)
            {
                // Select all controllers with the right name across all loaded plugins
                IEnumerable<Lazy<IController>> exports = _context.Container.GetExports<IController, IControllerMetaData>()
                    .Where(c =>
                           c.Metadata.ControllerName.ToLowerInvariant() == controllerName.ToLowerInvariant()
                    );

                // Iterate around our plugins in order to select the highest priority controller
                if (exports.Count() > 0)
                {
                    foreach (IPluginRegistration plugin in _context.Plugins)
                    {
                        foreach (Lazy<IController> export in exports)
                        {
                            if (Assembly.GetAssembly(export.GetType()).GetName().Name == plugin.AssemblyName)
                            {
                                return export.Value;
                            }
                        }
                    }

                    // Shouldn't really be reached
                    return exports.First().Value;
                }
            }

            // Controller not found, it must be in the main project
            return _defaultFactory.CreateController(requestContext, controllerName);
        }

        public void ReleaseController(IController controller)
        {
            IDisposable disposable = controller as IDisposable;

            if (disposable != null)
            {
                disposable.Dispose();
            }
        }

        #endregion
    }
}